package ats;

import java.util.Hashtable;
import java.util.Vector;

import salsa.language.Message;
import salsa.language.UniversalActor;
import salsa.naming.UAN;
import core.agents.ProfilingAgent;


public class ATSProfilingAgent implements ProfilingAgent {

	private double			threshold = 0.3;
	private	double			maxFLOPS = 0;
	public	double			getMaxFLOPS()			{ return maxFLOPS; }
	public	boolean 		isLight()			{ return getAvailableProcessing() > (maxFLOPS*threshold); }

	private	long			lastResetTime = System.currentTimeMillis();
	public	long			timeSinceLastReset()		{ return System.currentTimeMillis() - lastResetTime; }

	private	boolean			computing = true;
	public	boolean			isComputing()			{ return computing; }
	public	void			setComputing(boolean computing)	{ this.computing = computing; }

	public ATSProfilingAgent() {
		if ( System.getProperty("silent") == null) {
			System.err.println("Determining cpu performance:");
		}

		long startTime = System.currentTimeMillis();
		long l = 2, calculations = 300000;
		for (int i = 0; i < calculations; i++) l *= 2;
		maxFLOPS = calculations / (double)((double)(System.currentTimeMillis()-startTime)/1000.0);

		if ( System.getProperty("silent") == null) {
			System.err.println("CPU performance: " + maxFLOPS*2 + " flops");
		}

		maxFLOPS *= 2;
		//typically the real max is about twice the inital FLOPS reading
	}

	private Hashtable actorProfiles = new Hashtable();
	private long totalProcessed = 0;
	private long processedSinceStart = 0;

	public double getAvailableProcessing() {
		Vector profiles = getActorProfiles();

		double usedFLOPS = 0;
		for (int i = 0; i < profiles.size(); i++) {
			ActorProfile candidate = (ActorProfile)profiles.get(i);

			if (candidate.timeProcessing() > candidate.timeSending()) {
				usedFLOPS += maxFLOPS*((double)(candidate.timeProcessing()-candidate.timeSending())/timeSinceLastReset());
			}
		}
		return maxFLOPS-usedFLOPS;
	}

	public double getFLOPSUsedBy(ActorProfile candidate) {
		Vector profiles = getActorProfiles();

		double totalProcessingTime = 0;
		for (int i = 0; i < profiles.size(); i++) {
			ActorProfile current = (ActorProfile)profiles.get(i);

			if (current.timeProcessing() > current.timeSending()) {
				totalProcessingTime += (double)(current.timeProcessing()-current.timeSending())/(double)timeSinceLastReset();
			}
		}
		double candidateTime = 0;
		if (candidate.timeProcessing() > candidate.timeSending()) {
			candidateTime = (double)(candidate.timeProcessing()-candidate.timeSending())/(double)timeSinceLastReset();
		}
		return maxFLOPS * (candidateTime/totalProcessingTime);
	}

	Vector stealSources = new Vector();

	public int getDistinctSteals() {
		return stealSources.size();
	}

	public void gotStealFrom(String source) {
		if (!stealSources.contains(source)) stealSources.add(source);
	}


	public long processed()				{ return totalProcessed; }

	public Vector getActorProfiles() 		{ return new Vector(actorProfiles.values()); }
	public Vector getActorReferences() {
		Vector profiles = getActorProfiles();
		Vector references = new Vector();

		for (int i = 0; i < profiles.size(); i++) {
			references.add( UniversalActor.getReferenceByName( ((ActorProfile)profiles.get(i)).getUAN()) );
		}
		return references;
	}

	public void beginProcessed(UAN identifier, Message message) {
		totalProcessed++;
		processedSinceStart++;

		ActorProfile target = (ActorProfile)actorProfiles.get(identifier);
		if (target == null) return;  
    
		target.processedMessage();
		target.beginProcessing();
	}
	public void endProcessed(UAN identifier, Message message) { 
		ActorProfile target = (ActorProfile)actorProfiles.get(identifier);
		if (target == null) return;  
		target.endProcessing();
	}

	public void received(UAN identifier, Message message) {
		if (message.getSource().getUAN() != null) {
			ActorProfile target = (ActorProfile)actorProfiles.get(message.getSource().getUAN());
			if (target == null) return;
			else if (message.getTarget().getUAL() != null) {
				target.incrementSent( message.getTarget().getUAL().getHost() );
			}
		}

		ActorProfile target = (ActorProfile)actorProfiles.get(identifier);
		if (target == null) return;

		if (message.getTarget().getUAL() != null) {
			target.incrementReceived( message.getTarget().getUAL().getHost() );
		}
	}

	public void beginSend(UAN identifier, Message message) {
		ActorProfile target = (ActorProfile)actorProfiles.get(identifier);
		if (target == null) return;    		
		if (message.getSource() != null && target.getUAN().equals(message.getSource().getUAN())) return;

		target.incrementSent( message.getTarget().getUAL().getHost() );
		target.beginSending();
	}
	public void endSend(UAN identifier, Message message) {
		ActorProfile target = (ActorProfile)actorProfiles.get(identifier);
		if (target == null) return;    		
		if (message.getSource() != null && target.getUAN().equals(message.getSource().getUAN())) return;
		target.endSending();
	}

	/*
		The following create new actor profiles and manage their
		entering and exiting the theater
	*/
	public void addProfile(UAN identifier) 		{ actorProfiles.put(identifier, new ActorProfile(identifier)); }
	public void removeProfile(UAN identifier) 	{ actorProfiles.remove(identifier); }

	public void reset() {
		totalProcessed = 0;
		lastResetTime = System.currentTimeMillis();
		stealSources.clear();

		Vector profiles = getActorProfiles();
		for (int i = 0; i < profiles.size(); i++) { ((ActorProfile)profiles.get(i)).reset(); }
	}

	public long totalProcessed() { return processedSinceStart; }
}
