package ats;

import java.util.Hashtable;
import java.util.Vector;

import salsa.language.ServiceFactory;
import salsa.messaging.TheaterService;
import salsa.naming.UAN;

public class ActorProfile {
	private boolean scheduled = false;
	private long processed = 0;
	private long timeProcessing = 0;
	private long received = 0;
	private long sent = 0;
	private long timeSending = 0;
	private long lastResetTime = System.currentTimeMillis();
	private Hashtable targetList = new Hashtable();
	private Hashtable sourceList = new Hashtable();

	private TheaterService theaterService = ServiceFactory.getTheater();

	public long processed()			{ return processed; }
	public long timeProcessing()		{ return timeProcessing; }
	public long received()			{ return received; }
	public long sent()			{ return sent; }
	public long timeSending()		{ return timeSending; }
	public long lastResetTime()		{ return lastResetTime; }
	public boolean getScheduled()		{ return scheduled; }
	public void setScheduled(boolean s)	{ scheduled = s; }
	
	private UAN uan;

	public UAN getUAN() { 
		return uan; 
	}

	public ActorProfile(UAN uan) {
		this.uan = uan;
	}

	public void reset() {
		processed = 0;
		timeProcessing = 0;
		received = 0;
		sent = 0;
		timeSending = 0;
		lastResetTime = System.currentTimeMillis();

		sourceList = new Hashtable();
		targetList = new Hashtable();
	}

	public Vector getMessageTargets() {
		return new Vector(targetList.keySet());
	}


	public void processedMessage() { processed++; }

	long beginProcessingTime = 0;
	public void beginProcessing() {
		beginProcessingTime = System.currentTimeMillis();
	}
	public void endProcessing() {
		timeProcessing += System.currentTimeMillis()-beginProcessingTime;
	}

	long beginSendingTime = 0;
	public void beginSending() {
		beginSendingTime = System.currentTimeMillis();
	}
	public void endSending() {
		timeSending += System.currentTimeMillis()-beginSendingTime;
	}

	public int receivedFrom(String targetLocation) {
		int receivedFrom = 0;

		if (sourceList == null) sourceList = new Hashtable();
		Object rec = sourceList.get(targetLocation);
		if (rec != null) receivedFrom = ((Integer)rec).intValue();

		return receivedFrom;
	}

	public int sentTo(String targetLocation) {
		int sentTo = 0;

		if (targetList == null) targetList = new Hashtable();
		Object sent = targetList.get(targetLocation);
		if (sent != null) sentTo = ((Integer)sent).intValue();

		return sentTo;
	}

	public void incrementReceived(String targetLocation) {
		received++;

		if (sourceList == null) sourceList = new Hashtable();

		/* add the target location to our list of peers and number of messages received from them */
		Object receivedFrom = sourceList.get(targetLocation);
		if (receivedFrom != null) receivedFrom = new Integer( ((Integer)receivedFrom).intValue() + 1 );
		else receivedFrom = new Integer(1);

		sourceList.put(targetLocation, receivedFrom);
	}


	public void incrementSent(String targetLocation) {
		sent++;

		if (targetList == null) targetList = new Hashtable();

		/* add the target location to our list of peers and number of messages sent to them */
		Object sentTo = targetList.get(targetLocation);
		if (sentTo != null) sentTo = new Integer( ((Integer)sentTo).intValue() + 1 );
		else sentTo = new Integer(1);

		targetList.put(targetLocation, sentTo);
	}       
}
