package ntsC2C.core.language;


import salsa.language.Message;
import salsa.language.RunTime;
import salsa.language.ServiceFactory;
import salsa.language.UniversalActor;
import salsa.naming.UAL;
import salsa.naming.UAN;
import ntsC2C.IOTheater;
import ntsC2C.core.agents.ProfilingAgent;

public class AutonomousActor extends UniversalActor {
	private transient ProfilingAgent profilingAgent;

	public AutonomousActor() {
		super();
		profilingAgent = IOTheater.getProfilingAgent();
	}

	public AutonomousActor(UAN _uan) {
		super(_uan);
		profilingAgent = IOTheater.getProfilingAgent();
	}

	public AutonomousActor(UAL _ual) {
		super(_ual);
		profilingAgent = IOTheater.getProfilingAgent();
	}


	public void send(Message message) {
		if (getUAN() != null) profilingAgent.beginSend(getUAN(), message);
		super.send(message);
		if (getUAN() != null) profilingAgent.endSend(getUAN(), message);
	}

	private void writeObject(java.io.ObjectOutputStream out) throws Exception {
		out.defaultWriteObject();
	}

	private void readObject(java.io.ObjectInputStream in) throws Exception {
		in.defaultReadObject();

		profilingAgent = IOTheater.getProfilingAgent();
	}

/**
 * Begin AutonomousActor.State
 */
public abstract class State extends UniversalActor.State {
	private transient ProfilingAgent profilingAgent;

	public State(UAN uan, UAL ual) {
		super(uan, ual);

		profilingAgent = IOTheater.getProfilingAgent();
		if (uan != null) profilingAgent.addProfile(uan);
	}

	public synchronized void putMessageInMailbox(Message message) {
		if (getUAN() != null) profilingAgent.received(getUAN(), message);
		super.putMessageInMailbox(message);
	}

    public void run() {
		standardOutput = ServiceFactory.getOutput();
		standardError = ServiceFactory.getError();

		while ((!isDestroyed()) && (!isMigrating()) && (!isDead() || !mailbox.isEmpty()) ) {  
			currentMessage = getMessage();

			if (getUAN() != null) profilingAgent.beginProcessed(getUAN(), currentMessage);
			process(currentMessage);
			if (getUAN() != null) profilingAgent.endProcessed(getUAN(), currentMessage);

			RunTime.finishedProcessingMessage();
		}
    }

	public void destroy() {
		if (getUAN() != null)
		{ 
			System.out.println("In destroy");
			System.out.println("Destroying "+ getUAN());
			profilingAgent.removeProfile( getUAN() ); 
		}
		super.destroy();
	}

	private void writeObject(java.io.ObjectOutputStream out) throws Exception {
		if (getUAN() != null) profilingAgent.removeProfile( getUAN() );
		out.defaultWriteObject();
	}

	private void readObject(java.io.ObjectInputStream in) throws Exception {
		in.defaultReadObject();

		profilingAgent = IOTheater.getProfilingAgent();
		if (getUAN() != null) profilingAgent.addProfile( getUAN() );
	}
}
/**
 * End AutonomousActor.State
 */
}
