package ntsC2C.pslb;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.UnknownHostException;



public class DelaySensor
{

	private static int port;
	private ServerSocket sensorSocket;

	public int getPort() 		{ return DelaySensor.port; }
	public void setPort(int _port)	{ DelaySensor.port = _port; }
	
	public DelaySensor()
	{
		//well know port for the delay Sensor unless specified otherwise
		DelaySensor.port = 5001;
	}
	
	public void startSensor()
	{
		if (sensorSocket == null) {
			//Initialize the port
			String propertyPort = null;
			propertyPort = System.getProperty("port");
			if (propertyPort != null) DelaySensor.port = Integer.parseInt(propertyPort);

			// Create a server socket.
			try {
				sensorSocket = new ServerSocket( DelaySensor.port );
			} catch (IOException e) {
				System.err.println("Delay sensor error:");
				System.err.println("\tCould not start sensor.");
				System.err.println("\tException: " + e);
			}
			
			// Check to see what the port actually is.
			if (DelaySensor.port == 0) {
				DelaySensor.port = sensorSocket.getLocalPort();
			}
			
			final ServerSocket passive = sensorSocket;
			
			// Accept connections.
			Runnable listenLoop = new Runnable() {
				public void run() {
					int i = 0;
					while (true) {
						try {
							final Socket incoming = passive.accept();
							Runnable handlerThread = new Runnable() {
								public void run() {
									processRequest(incoming);
								}
							};
							new Thread(handlerThread, "Handler Loop " + i).start();
						} catch (IOException e) {
							System.err.println("Delay Sensor error: ");
							System.err.println("\tFailed to accept connection.");
							System.err.println("\teException: " + e);
						}
	
						i++;
					}
				}
			};
	
			// Start the listening thread.
			new Thread( listenLoop, "Listen Loop Thread" ).start();
		}

	}
	
	public void processRequest(Socket socket) 
	{
		//Display peer details
		System.out.println("Received ping request from "+ socket.getInetAddress()+":"+socket.getLocalPort());
		
		try {
			ObjectInputStream ois = new ObjectInputStream(socket.getInputStream());
			ObjectOutputStream oos = null;
			
			while(true){
			
				//Read one byte and send back one byte
				ois.readByte();
				
				if(oos == null){
					oos = new ObjectOutputStream(socket.getOutputStream());
				}
				
				oos.writeByte(0);
				oos.flush();
			}
			
			
		}catch(EOFException e)
		{
			System.out.println("Socket reset by peer");
			//e.printStackTrace();
		} 
		catch (Exception e) {
			e.printStackTrace();
		}
						
	}
	
	
	public static void main(String[] args)
	{
	
		//Check the argument
		if (args.length > 1)
		{
			System.out.println("Usage: DelaySensor <port>");
			return;
		}
		
		//Create Delay sensor
		DelaySensor sensor = new DelaySensor();
		
		
		if(args.length == 1)
		{
			try {
				DelaySensor.port = Integer.parseInt( args[0] );
			} catch (NumberFormatException e) {
					System.err.println( "DelaySensor Service error:" );
					System.out.println( "\tBad port: " + args[0] );
					return;
			}
		}
	
		sensor.startSensor();
		
		try {
			String localInfo = "rmsp://" + InetAddress.getLocalHost().getHostAddress() + ":" + DelaySensor.port;
			if ( System.getProperty("silent") == null ) System.err.println("Delay Sensor started at: " + localInfo);
		} catch (UnknownHostException e) {
					System.err.println("Delay Sensor error: ");
					System.err.println("\tUnknown local host");
					System.err.println("\tException: " + e);
		}
	
	}
	

}