package ntsC2C.pslb;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.Vector;

import ntsC2C.core.agents.ProfilingAgent;
import salsa.language.ActorReference;
import salsa.language.Message;
import salsa.language.UniversalActor;
import salsa.naming.UAN;

public class PSLBProfilingAgent implements ProfilingAgent{


	private double flops=0;	
	private int numTrials = 10;
	private int pingTrials = 10;
	int delaySensorPort = 5001;
	private int calculations = 30000;

	static Hashtable actorProfiles = new Hashtable();

	public PSLBProfilingAgent(){}



	private long totalProcessed = 0;
	private long processedSinceStart = 0;

	// supposed to be cycles/s (The processing value)

	private double processing = 0;
	public double getProcessing() 			{ return processing; }
	public void setProcessing(double processing)	{ this.processing = processing; }
	public long processed()				{ return totalProcessed; }

	public Vector getActorProfiles() 		{ return new Vector(actorProfiles.values()); }
	public Vector getActorReferences() {
		Vector profiles = getActorProfiles();
		Vector references = new Vector();

		for (int i = 0; i < profiles.size(); i++) {
			references.add( new UniversalActor( ((ActorProfile)profiles.get(i)).getUAN()) );
		}
		return references;
	}

	int maxMailboxSize = 0;
	int currentMailboxSize = 0;
	public boolean isLight() 			{ return (currentMailboxSize <= maxMailboxSize); }

	boolean computing = true;
	public boolean isComputing()			{ return computing; }
	public void setComputing(boolean computing)	{ this.computing = computing; }


	public void beginProcessed(UAN identifier, Message message) {
		totalProcessed++;
		processedSinceStart++;

		ActorProfile target = (ActorProfile)actorProfiles.get(identifier);
		if (target == null) return;  
    
		target.processedMessage();
	}
	public void endProcessed(UAN identifier, Message message) { 
		currentMailboxSize--;
	}

	public void received(UAN identifier, Message message) {
		ActorProfile target = (ActorProfile)actorProfiles.get(identifier);
		if (target == null) return;

		if (message.getTarget().getUAL() != null) {
			target.incrementReceived( message.getTarget().getUAL().getLocation() );
		}

		currentMailboxSize++;
		if (currentMailboxSize > maxMailboxSize) maxMailboxSize = currentMailboxSize;
	}

	public void beginSend(UAN identifier, Message message) {
		ActorProfile target = (ActorProfile)actorProfiles.get(identifier);
		if (target == null) return;    		
		target.incrementSent( message.getTarget().getUAL().getLocation() );
	}
	public void endSend(UAN identifier, Message message) { }

	public void reset() {
		totalProcessed = 0;
		maxMailboxSize = 0;
		currentMailboxSize = 0;

		Vector profiles = getActorProfiles();
		for (int i = 0; i < profiles.size(); i++) 
		 
			if ( (ActorProfile)profiles.get(i) != null )
				((ActorProfile)profiles.get(i)).reset(); 
	}

	public long totalProcessed() { return processedSinceStart; }



///

	int getLoad(){ return actorProfiles.size(); }
	
	
	public long getPeerDelay(String host)
	{
		long delay=0;
		
		int port = delaySensorPort;
		Socket sensorSocket = null;
		ObjectInputStream ois = null;
		ObjectOutputStream oos = null;
		
		long startTime, endTime, totalTime, avgTime;
		
		// Opening a socket with the delay sensor at target Peer
		try {
			sensorSocket = new Socket(host,port);
			
			//disable the nagle algorithm since we are sending only one byte
			sensorSocket.setTcpNoDelay(false);
			
			oos = new ObjectOutputStream(sensorSocket.getOutputStream());
						
		} catch (UnknownHostException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		// Ping several times
		totalTime = 0;
		for (int i=0; i< pingTrials; i++)
		{
			startTime = System.currentTimeMillis();
			
			try {
				
				oos.write(0);
				oos.flush();				
				if(ois == null){
					ois = new ObjectInputStream(sensorSocket.getInputStream() ); 
				}				
				//Read the byte back
				ois.readByte();				
				
			} catch (IOException e1) {
				e1.printStackTrace();
			}
			endTime = System.currentTimeMillis();
			System.out.println("Round Trip Time "+i+" with host "+host+": "+ (endTime-startTime)+" ms");
			
			//Skip the first measurement
			if(i!= 0) totalTime += endTime-startTime;
		}
		avgTime = totalTime/(pingTrials-1);
		
		System.out.println("Average Trip Time: "+ avgTime+" ms");
		
		try {
			oos.close();
			ois.close();
			sensorSocket.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
				
		return avgTime;	
	}
	
	public long getCpuUsage()
	{
		long startTime, endTime, avgElapsedTime, totalTime;
		int l;
		
		totalTime = 0;
		for(int i=0; i<numTrials; i++){
			
			startTime = System.currentTimeMillis();
			
			// Perform some CPU intensive operation
			
			l = 1;
			for(int k=0; k<1000000; k++) l *=2;
					
				endTime = System.currentTimeMillis();
			
				totalTime += endTime - startTime;
			}
		
			// Returns the average value over the number of trials
		
			return (long)(totalTime/(long)numTrials);
	}
	
	public long getFreeMemory()
	{
		
		long freeMem;
		Runtime rt = Runtime.getRuntime();
				
		freeMem = 0;
		for(int i=0; i<numTrials; i++)
		{
			freeMem += rt.freeMemory();		
		}	
		
		//Return the average free JVM memory over the number of trials
		
		return (long) (freeMem/(long)numTrials);
	}
	
	
	public void calculateFLOPS() {
		
		long startTime = System.currentTimeMillis();
		long l = 1;
		for (int i = 0; i < calculations; i++) l *= 2;
		flops = (1.0/4.0)*flops + (3.0/4.0)*(double)calculations / (double)((double)(System.currentTimeMillis()-startTime)/(double)1000);

		setProcessing(flops);
	}
	
	public double getFLOPS() { return flops; }
	

	/*
		The following create new actor profiles and manage their
		entering and exiting the theater
	*/
	public void addProfile(UAN identifier) 		
	{ 
		//System.out.println("\t Actor enters <<"+identifier+">>");
		actorProfiles.put(identifier, new ActorProfile(identifier)); 
		//System.out.println("========Size ="+ actorProfiles.size());
		
	}
	
	public void removeProfile(UAN identifier) 	{
		//System.out.println("\t Actors exits <<"+identifier+">>");
		actorProfiles.remove(identifier); 
	}

	public synchronized Vector getCandidates(int numActors)
	{
		Random rg = new Random(System.currentTimeMillis());
		Vector candidatesID = new Vector();
		Vector candidatesRef = new Vector();
		Vector actors = getTargets();
		int size = actors.size();
		UAN id;
		int r;
		ActorProfile profile;
		
		//returns an empty vector
		if(numActors > actorProfiles.size()){
			System.out.println("<profilingAgent> incosistent state at getCandidates!!!");
			return candidatesRef;
		}
		
		int i = 0; 
		//System.out.println("\tThe actors picked are:");
		while(i<numActors)
		{
			//Pick randomly candidates from the list of actors
			
			r = rg.nextInt(size);
			id = (UAN)actors.get(r);
			if (!candidatesID.contains(id))
			{
				candidatesID.add(id);
				//System.out.println("\t"+id);
				candidatesRef.add( UniversalActor.getReferenceByName(id) );
				i++;		
			}		
		}		
		
		return candidatesRef;
	}

	public Vector getTargets()
	{
		//System.out.println("In getTargets()");
		
		Vector targets = new Vector();
			
		Iterator iterEntries = actorProfiles.entrySet().iterator();
		while(iterEntries.hasNext()){
				
			Map.Entry mapEntry = (Map.Entry)iterEntries.next() ;
			//System.out.println(mapEntry.getKey());		
			targets.addElement((UAN)mapEntry.getKey());		
		}				
		return targets;	
	}

	public ActorReference getCandidate()	
	{
			Random rg = new Random(System.currentTimeMillis());
			
			Vector targets = getTargets();
			String id = (String)targets.get(rg.nextInt(targets.size()));
			//System.out.println("After getting the id in getCandidate");
			
			return getReference(id);
	}
	
	public ActorReference getReference(String id)
	{
		ActorProfile candidate = (ActorProfile)actorProfiles.get(id);
			
		if (candidate != null ){
			return UniversalActor.getReferenceByName( candidate.getUAN());
		}
		return null;		
	
	}


}


