package theater;

import salsa.language.ActorReference;
import salsa.language.ServiceFactory;
import wwc.messaging.Theater;
import core.agents.ProfilingAgent;

public class IOTheater extends Theater {
	static ActorReference protocolActor = null;
	static ActorReference decisionActor = null;
	static ProfilingAgent profilingAgent = null;

	public static synchronized ActorReference getProtocolActor() {
		if (protocolActor == null) {
			//Check for the protocol actor
			//default = io.ats.ATSProtocolActor
			String className = System.getProperty( "protocol" );  
			if (className == null) className = "ntsP2P.NTSProtocolActor";

			try {
				protocolActor = ServiceFactory.createActor(className, "io/protocolActor");
			} catch (Exception e) {
				System.err.println("Error creating protocol actor: " + className);
				System.err.println("\t" + e);
				e.printStackTrace();
			}
		}
		return protocolActor;
	}

	public static synchronized ActorReference getDecisionActor() {
		if (decisionActor == null) {
			//Check for the decision actor
			//default = io.ats.ATSDecisionActor
			String className = System.getProperty( "decision" );  
			if (className == null) className = "ntsP2P.NTSDecisionActor";

			try {
				decisionActor = ServiceFactory.createActor(className, "io/decisionActor");
			} catch (Exception e) {
				System.err.println("Error creating decision actor: " + className);
				System.err.println("\t" + e);
				e.printStackTrace();
			}
		}
		return decisionActor;
	}

	public static synchronized ProfilingAgent getProfilingAgent() {
		if (profilingAgent == null) {
			//Check for the profiling agent
			//default = io.ats.ATSProfilingAgent
			String className = System.getProperty( "profiling" );  
			if (className == null) className = "ntsP2P.NTSProfilingAgent";

			try {
				profilingAgent = (ProfilingAgent)Class.forName( className ).newInstance();
			} catch (Exception e) {
				System.err.println("Error creating profiling agent: " + className);
				System.err.println("\t" + e);
				e.printStackTrace();
			}
		}
		return profilingAgent;
	}

	public IOTheater() {
		String[] args = { "0" };
                Theater.main(args);

		getProfilingAgent();
		getDecisionActor();
		getProtocolActor();
	}

	public static void main(String[] arguments) {
		if (arguments.length == 0) {
			arguments = new String[1];
			arguments[0] = "0";
		}

		Theater.main(arguments);

		getProfilingAgent();
		getDecisionActor();
		getProtocolActor();
	}
}
